@@ -3,7 +3,7 @@ package Markdent::Role::AnyParser;
 use strict;
 use warnings;
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 use Moose::Role;
 
@@ -15,6 +15,21 @@ has handler => (
     required => 1,
 );
 
+sub _send_event {
+    my $self = shift;
+
+    $self->handler()->handle_event( $self->_make_event(@_) );
+}
+
+sub _make_event {
+    my $self  = shift;
+    my $class = shift;
+
+    my $real_class = $class =~ /::/ ? $class : 'Markdent::Event::' . $class;
+
+    return $real_class->new(@_);
+}
+
 sub _detab_text {
     my $self = shift;
     my $text = shift;
@@ -67,10 +82,6 @@ replaces tabs with spaces.
 
 This class does the L<Markdent::Role::DebugPrinter> role.
 
-=head1 AUTHOR
-
-Dave Rolsky, E<gt>autarch@urth.orgE<lt>
-
 =head1 BUGS
 
 See L<Markdent> for bug reporting details.
