@@ -3,7 +3,9 @@ package Markdent::Role::EventsAsMethods;
 use strict;
 use warnings;
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
+
+use Scalar::Util qw( blessed );
 
 use namespace::autoclean;
 use Moose::Role;
@@ -16,10 +18,7 @@ sub handle_event {
 
     my $meth = $event->event_name();
 
-    my %p = %{ $event->attributes() };
-    delete @p{ grep {/^!/} keys %p };
-
-    $self->$meth(%p);
+    $self->$meth( $event->kv_pairs_for_attributes() );
 }
 
 1;
@@ -54,10 +53,6 @@ attributes will not be passed to the method.
 
 This class does the L<Markdent::Role::Handler> role.
 
-=head1 AUTHOR
-
-Dave Rolsky, E<gt>autarch@urth.orgE<lt>
-
 =head1 BUGS
 
 See L<Markdent> for bug reporting details.
